setClass("matFile", representation(
		ptr = "externalptr",
		file = "character",
		mode = "character"))

setMethod("initialize", "matFile",
function(.Object, file, mode="u", ...) {
	mat <- callNextMethod(.Object, ...)
	if (missing(file))
		return(mat)
	ptr <- .Call(RmatOpen, file, mode)
	if (is.null(ptr))
		stop("error opening matFile: ", file)
	mat@ptr <- ptr
	mat@file <- file
	mat@mode <- mode
	mat
})

#setMethod("close", "matFile",
close.matFile <-
function(con, ...) {
	.Call(RmatClose, con@ptr)
	invisible(NULL)
}

setMethod("print", "matFile",
function(x, ...) {
	if (is.nullPtr(x@ptr))
		print(NULL, ...)
	else
	{
		cat("matFile '", x@mode, "'\n", sep="")
		print(x@file, ...)
	}
	invisible(x)
})
setMethod("show", "matFile",
function(object) {
	print(object)
	invisible(NULL)
})

setMethod("names", "matFile",
function(x) {
	.Call(RmatGetDir, x@ptr)
})

setMethod("[[", "matFile",
function(x, i, j, ..., exact=TRUE) {
	if (missing(i) || !missing(j))
		stop('incorrect number of subscripts')
	if (!exact)
		i <- ns[[pmatch(i, ns <- names(x))]]
	if (is.na(i))
		return(NULL)
	mxArray(.Call(RmatGetVariable, x@ptr, i))
})

setMethod("[[<-", "matFile",
function(x, i, j, ..., global=FALSE, value) {
	if (missing(i) || !missing(j))
		stop('incorrect number of subscripts')
	if (!.Call(RmatSetVariable, x@ptr, i, mxArrayPtr(value), global))
		stop("failed to set matFile variable")
	x
})

setMethod("$", "matFile", function(x, name) `[[`(x, name))
setMethod("$<-", "matFile", function(x, name, value) `[[<-`(x, name, value=value))

setAs("matFile", "list",
function(from) {
	to <- list()
	for (n in names(from))
		to[[n]] <- from[[n]]
	to
})

setMethod("[", "matFile",
function(x, i, j, ..., recursive=FALSE, drop=FALSE) {
	if (!missing(i))
	{
		if (!missing(j))
			stop("incorrect number of subscripts")
		x <- x[[i,...]][recursive=recursive, drop=drop]
	}
	else
	{
		x <- as(x, 'list')
		if (recursive)
			x[] <- lapply(x, `[`, recursive=recursive, drop=drop)
	}
	x
})

setMethod("[<-", "matFile",
function(x, i, j, ..., value) {
	if (missing(i) || !missing(j))
		stop('incorrect number of subscripts')
	x[[i,...]] <- as.matlab(value)
	x
})

matRead <- 
function(file) {
	mat <- new("matFile", file, "r")
	r <- list()
	while (!is.null(x <- .Call(RmatGetNextVariable, mat@ptr)))
		r[[attr(x,'name')]] <- mxArray(x)
	close(mat)
	r
}

matWrite <-
function(file, value) {
	mat <- new("matFile", file, "w")
	for (i in seq_along(value))
		mat[names(value[i])] <- value[[i]]
	close(mat)
}

matFile <-
function(file, mode, value) {
	if (missing(mode))
		if (missing(value))
			matRead(file)
		else
			matWrite(file, value)
	else
		if (missing(value))
			new("matFile", file, mode)
		else
			stop("only one of mode and value may be specified")
}

`matFile<-` <-
function(file, value) {
	matWrite(file, value)
	file
}
