setClass("matlabEngine", representation(
		ptr = "externalptr",
		cmd = "character"))

setMethod("initialize", "matlabEngine",
function(.Object, cmd, ...) {
	eng <- callNextMethod(.Object, ...)
	if (missing(cmd))
		return(eng)
	ptr <- .Call(RmatlabEngineOpen, cmd)
	if (is.null(ptr))
		stop("error opening matlabEngine: ", cmd)
	eng@ptr <- ptr
	eng@cmd <- cmd
	eng
})

#setMethod("close", "matlabEngine",
close.matlabEngine <-
function(con, ...) {
	.Call(RmatlabEngineClose, con@ptr)
	invisible(NULL)
}

setMethod("print", "matlabEngine",
function(x, ...) {
	if (is.nullPtr(x@ptr))
		print(NULL, ...)
	else
		cat("matlabEngine", x@cmd, "\n")
	invisible(x)
})
setMethod("show", "matlabEngine",
function(object) {
	print(object)
	invisible(NULL)
})

setMethod("[[", "matlabEngine",
function(x, i, j, ...) {
	if (missing(i) || !missing(j))
		stop('incorrect number of subscripts')
	if (is.na(i))
		return(NULL)
	mxArray(.Call(RmatlabEngineGetVariable, x@ptr, i))
})

setMethod("[[<-", "matlabEngine",
function(x, i, j, ..., value) {
	if (missing(i) || !missing(j))
		stop('incorrect number of subscripts')
	if (!.Call(RmatlabEnginePutVariable, x@ptr, i, mxArrayPtr(value)))
		stop("failed to set matlabEngine variable")
	x
})

setMethod("[", "matlabEngine",
function(x, i, j, ..., recursive=FALSE, drop=FALSE) {
	if (missing(i) || !missing(j))
		stop('incorrect number of subscripts')
	x[[i,...]][recursive=recursive, drop=drop]
})

setMethod("[<-", "matlabEngine",
function(x, i, j, ..., value) {
	if (missing(i) || !missing(j))
		stop('incorrect number of subscripts')
	x[[i,...]] <- as.matlab(value)
	x
})

setGeneric("evaluate", function(object, ...) standardGeneric("evaluate"))
setMethod("evaluate", "matlabEngine",
function(object, command, ..., output=FALSE, nargout=1, tmp.var.prefix = "Reval_") {
	if (isTRUE(output))
		output <- 4096
	command <- as.character(command)
	args <- list(...)
	if (length(args))
	{
		in.vars <- paste(tmp.var.prefix, "in", seq_along(args), sep="")
		for (i in seq_along(args))
		{
			if (is.name(args[[i]]))
				in.vars[[i]] <- as.character(args[[i]])
			else
				object[in.vars[[i]]] <- args[[i]]
		}
		command <- paste(command, "(", paste(in.vars, collapse=","), "); clear ", tmp.var.prefix, "in*", sep="")
	}
	out.vars <- list()
	if ((!missing(nargout) || length(args)) && nargout)
	{
		out.vars <- paste(tmp.var.prefix, "out", seq_len(nargout), sep="")
		command <- paste("clear ", tmp.var.prefix, "out*; [", paste(out.vars, collapse=","), "] = ", command, sep="")
	}
	res <- .Call(RmatlabEngineEval, object@ptr, command, output)
	if (identical(res, FALSE))
		stop('eval(', command, ') failed')
	if (length(out.vars))
	{
		out <- list()
		for (i in seq_along(out.vars))
			out[[i]] <- object[[out.vars[[i]]]]
		if (output)
			out[['output']] <- res
		else if (length(out) == 1)
			out <- out[[1]]
		return(out)
	}
	else if (output)
		return(res)
	else if (missing(nargout))
		return(invisible(eng[['ans']]))
})

setMethod("$", "matlabEngine", 
function(x, name) {
	if (!is.null(y <- x[[name]]))
		return(y)
	function(...) evaluate(x, name, ...)
});

setMethod("$<-", "matlabEngine", function(x, name, value) `[[<-`(x, name, value=value))

matlabEngine <- function(cmd = getOption("matlabBin"), args = c("-nosplash","-nojvm")) 
	new("matlabEngine", paste(c(cmd, args), collapse=" "))
