####################################### mxClassID
setClass("mxClassID", contains = "factor")

setMethod("initialize", "mxClassID",
function(.Object, x = NA) {
	if (is.integer(x))
		x <- mxClassNames[x]
	callNextMethod(.Object, factor(x, mxClassNames))
})
setAs("character", "mxClassID", function(from) new("mxClassID", from))
setAs("integer",   "mxClassID", function(from) new("mxClassID", from))

setMethod("print", "mxClassID", function(x, quote=FALSE, ...) print(as.character(x), quote=quote))
setMethod("show", "mxClassID", function(object) { print(object); invisible(NULL) })


####################################### mxArray
setClass("mxArray", representation(
		ptr = "externalptr", 
		classID = "mxClassID",
		.dim = "integer",
		.length = "integer",
		global = "logical"))

is.mxArrayPtr <- function(x) is.externalptr(x) && .Call(RmxIsArray, x)
is.mxArray <- function(x) inherits(x, "mxArray")

setMethod("initialize", "mxArray", 
function(.Object, dim, classID, ptr) {
	mx <- .Object #callNextMethod(.Object, ...)
	if (!missing(ptr))
	{
		if (!is.mxArrayPtr(ptr))
			stop("mxArray creation failed")
		classID <- .Call(RmxGetClass, ptr)
		dim <- .Call(RmxGetDimensions, ptr)
		length <- .Call(RmxGetLength, ptr)
		global <- .Call(RmxIsGlobal, ptr)
	}
	else if (!missing(dim))
	{
		dim <- as.integer(dim)
		length <- as.integer(prod(dim))
	}
	if (!missing(ptr))	mx@ptr	   <- ptr
	if (!missing(classID))	mx@classID <- as(classID, 'mxClassID')
	if (!missing(dim))	
	{
		mx@.dim    <- dim
		mx@.length <- length
	}
	mx
})

setMethod("length", "mxArray", function(x) x@.length)
setMethod("dim", "mxArray", function(x) x@.dim)
setMethod("dim<-", "mxArray", 
function(x, value) {
	if (.Call(RmxSetDimensions, x@ptr, value))
	{
		x@.dim <- .Call(RmxGetDimensions, x@ptr)
		x@.length <- .Call(RmxGetLength, x@ptr)
	}
	x
})

mxGlobal <- 
function(mx, global) {
	if (!is.mxArray(mx))
		stop('must be mxArray')
	if (missing(global))
		global <- mx@global
	else if (!is.null(global <- .Call(RmxSetGlobal, mx@ptr, global)))
		mx@global <- global
	global
}

setMethod("print", "mxArray",
function(x, ...) {
	if (is.nullPtr(x@ptr))
		print(NULL, ...)
	else
	{
		cat(x@.dim, sep="x")
		cat("", as.character(x@classID), class(x), "\n")
	}
	invisible(x)
})
setMethod("show", "mxArray",
function(object) {
	print(object)
	invisible(NULL)
})

setAs("mxArray", "raw", function(from)
	.Call(RmxGetRaw, from@ptr))

mxIndex <-
function(i, j, ...) {
	if (missing(i)) i <- NULL
	if (missing(j)) j <- NULL
	as.integer(c(i,j,...))
}

mxCopy <- function(mx) {
	if (!is.mxArray(mx) || !is.mxArrayPtr(mx@ptr))
		return(mx)
	mx@ptr <- .Call(RmxDuplicateArray, mx@ptr)
	mx
}

mxArrayPtr <-
function(x) {
	if (is.mxArray(x))
		x <- x@ptr
	else if (!is.null(x))
		stop("requires mxArray")
	x
}

####################################### mxDataArray : mxArray
setClass("mxDataArray", contains = "mxArray")

mxDataType <-
function(x)
	switch(as.character(x@classID),
			cell = "list",
			logical = "logical",
			char = "character",
			double = "double",
			single = "double",
			int8 = "integer",
			uint8 = "integer",
			int16 = "integer",
			uint16 = "integer",
			int32 = "integer")

setMethod("[[", "mxDataArray",
function(x, i, j, ...)
	.Call(RmxGetElement, x@ptr, mxIndex(i,j,...)))

setMethod("[[<-", "mxDataArray",
function(x, i, j, ..., value) {
	.Call(RmxSetElement, x@ptr, mxIndex(i,j,...), value)
	x
})

mxDataSet <-
function(mx, data) {
	n <- .Call(RmxGetLength, mx@ptr)
	if (n != length(data))
	{
		#stop("incorrect data length for mxDataArray")
		if (n %% length(data))
			warning('number of items to replace is not a multiple of replacement length')
		data <- rep(data, length.out=n)
	}
	data <- as.vector(data, mode=mxDataType(mx))
	# try fast way:
	if (!is.NA(.Call(RmxSetVector, mx@ptr, data)))
		return(mx)
	# fall through to slow way:
	for (i in seq_len(n))
		mx[[i]] <- data[[i]]
}

setAs("mxDataArray", "vector",
function(from) {
	# try fast way:
	to <- .Call(RmxGetVector, from@ptr)
	if (!is.NA(to))
		return(to)
	# fall through to slow way:
	n <- length(from)
	to <- vector(mxDataType(from), n)
	for (i in seq_len(n))
	{
		e <- from[[i]]
		if (!is.null(e))
			to[[i]] <- e
	}
	return(to)
},
function(from, value) {
	mxDataSet(from, value)
	from
})

setAs("mxDataArray", "array",
function(from) {
	to <- as(from, 'vector')
	dim(to) <- dim(from)
	to
},
function(from, value) {
	dim(from) <- dim(value)
	as(from, "vector") <- value
	from
})

setAs("array", "mxDataArray", 
function(from) {
	c <- switch(typeof(from),
		list = "Cell",
		logical = "Logical",
		character = "Char",
		numeric = "Numeric",
		complex = "Numeric",
		integer = "Integer")
	if (is.null(c))
		step("unsupported array type")
	else
		new(paste("mx", t, "Array", sep=""), data=from)
})

setMethod("[", "mxDataArray",
function(x, i, j, ..., drop=FALSE) {
	if (!missing(i))
		y <- x[[mxIndex(i,j,...)]]
	else
	{
		y <- as(x, "array")
		if (drop)
			y <- base::drop(y)
	}
	y
})

setMethod("[<-", "mxDataArray",
function(x, i, j, ..., value) {
	if (!missing(i))
		x[[mxIndex(i,j,...)]] <- value
	else
		mxDataSet(x, value)
	x
})


####################################### mxCellArray : mxDataArray : mxArray
setClass("mxCellArray", contains = "mxDataArray")

setMethod("initialize", "mxCellArray",
function(.Object, data, classID='cell', ...) {
	mx <- callNextMethod(.Object, classID=classID, ...)
	if (is.nullPtr(mx@ptr) && !(is.empty(mx@.dim) && missing(data)))
	{
		if (is.empty(dim <- mx@.dim))
			dim <- dim.vector(data)
		ptr <- .Call(RmxCreateCellArray, dim)
		mx <- callNextMethod(mx, ptr=ptr)
		if (!missing(data))
			mxDataSet(mx, data)
	}
	mx
})

setMethod("[[", "mxCellArray",
function(x, i, j, ...)
	mxArray(.Call(RmxGetElement, x@ptr, mxIndex(i,j,...))))

setMethod("[[<-", "mxCellArray",
function(x, i, j, ..., value) {
	.Call(RmxSetElement, x@ptr, mxIndex(i,j,...), mxArrayPtr(value))
	x
})

setAs("list", "mxCellArray", function(from)
	new("mxCellArray", data=from))

setMethod("[", "mxCellArray",
function(x, i, j, ..., recursive=FALSE, drop=FALSE) {
	if (!missing(i))
	{
		x <- x[[mxIndex(i,j,...)]]
		if (is.null(x))
			return(x)
	}
	x <- as(x, "array")
	if (drop)
		x <- base::drop(x)
	if (recursive)
		x[] <- lapply(x, `[`, recursive=recursive, drop=drop)
	x
})

setMethod("[<-", "mxCellArray",
function(x, i, j, ..., value) {
	if (!missing(i))
		x[[mxIndex(i,j,...)]] <- as.matlab(value)
	else
		mxDataSet(x, value)
	x
})


####################################### mxNumericArray : mxDataArray : mxArray
setClass("mxNumericArray", contains = "mxDataArray", representation(
			complex = "logical",
			sparse = "logical"))

setMethod("initialize", "mxNumericArray",
function(.Object, data, complex, sparse=FALSE, ...) {
	mx <- callNextMethod(.Object, ...)
	if (is.nullPtr(mx@ptr) && (!missing(data) || !is.empty(mx@.dim) && !is.na(mx@classID)))
	{
		if (sparse)
			stop("unsupported sparse mxNumericArray creation")
		if (is.empty(dim <- mx@.dim))
			dim <- dim.vector(data)
		if (is.na(classID <- as.character(mx@classID)))
			classID <- switch(typeof(data),
				integer = "int32",
				double = "double",
				complex = "double",
				'unknown')
		classID <- as(classID, 'mxClassID')
		if (missing(complex))
			complex <- !missing(data) && is.complex(data)
		ptr <- .Call(RmxCreateNumericArray, dim, classID, complex)
		mx <- callNextMethod(mx, ptr=ptr)
		if (!missing(data))
			mxDataSet(mx, data)
	}
	if (!is.nullPtr(mx@ptr))
	{
		mx@complex <- .Call(RmxIsComplex, mx@ptr)
		mx@sparse <- .Call(RmxIsSparse, mx@ptr)
	}
	mx
})

setAs("integer", "mxNumericArray", function(from)
	new("mxNumericArray", data=from, classID='int32'))
setAs("numeric", "mxNumericArray", function(from)
	new("mxNumericArray", data=from, classID='double'))
setAs("complex", "mxNumericArray", function(from)
	new("mxNumericArray", data=from, classID='double'))


####################################### mxLogicalArray : mxDataArray : mxArray
setClass("mxLogicalArray", contains = "mxDataArray", representation(
			sparse = "logical"))

setMethod("initialize", "mxLogicalArray",
function(.Object, data, sparse=FALSE, classID='logical', ...) {
	mx <- callNextMethod(.Object, classID=classID, ...)
	if (is.nullPtr(mx@ptr) && (!is.empty(mx@.dim) || !missing(data)))
	{
		if (sparse)
			stop("unsupported sparse mxLogicalArray creation")
		if (is.empty(dim <- mx@.dim))
			dim <- dim.vector(data)
		ptr <- .Call(RmxCreateLogicalArray, dim)
		mx <- callNextMethod(mx, ptr=ptr)
		if (!missing(data))
			mxDataSet(mx, data)
	}
	if (!is.nullPtr(mx@ptr))
	{
		mx@sparse <- .Call(RmxIsSparse, mx@ptr)
	}
	mx
})

setAs("logical", "mxLogicalArray", function(from) 
	new("mxLogicalArray", data=from))


####################################### mxCharArray : mxDataArray : mxArray
setClass("mxCharArray", contains = "mxDataArray")

setMethod("initialize", "mxCharArray",
function(.Object, data, classID="char", ...) {
	mx <- callNextMethod(.Object, classID=classID, ...)
	if (is.nullPtr(mx@ptr) && (!is.empty(mx@.dim) || !missing(data)))
	{
		dim <- mx@.dim
		if (is.empty(dim) && (is.vector(data, 'character') || is.character(data) && any(sapply(data, nchar) > 1)))
		{
			ptr <- .Call(RmxCreateStrings, data)
			mx <- callNextMethod(mx, ptr=ptr)
		}
		else
		{
			if (is.empty(dim))
				dim <- dim.vector(data)
			ptr <- .Call(RmxCreateCharArray, dim)
			mx <- callNextMethod(mx, ptr=ptr)
			if (!missing(data))
				mxDataSet(mx, data)
		}
	}
	mx
})

setAs("mxCharArray", "character", function(from) 
	.Call(RmxGetString, from@ptr))
setAs("character", "mxCharArray", function(from)
	new("mxCharArray", data=from))


####################################### mxStructArray : mxArray
setClass("mxStructArray", contains = "mxArray", representation(
			fields = "character"))

mxStructSet <-
function(mx, data) {
	n <- .Call(RmxGetLength, mx@ptr)
	f <- .Call(RmxGetFields, mx@ptr)
	if (n*f != length(data))
	{
		#stop("incorrect data length for mxStructArray")
		if (n*f %% length(data))
			warning('number of items to replace is not a multiple of replacement length')
		data <- rep(data, length.out=n*f)
	}
	dim(data) <- c(f,n)
	for (i in seq_len(n))
		for (j in seq_len(f))
			mx[[i,field=j]] <- data[[j,i]]
}

setMethod("initialize", "mxStructArray",
function(.Object, data, fields, classID='struct', ...) {
	mx <- callNextMethod(.Object, classID=classID, ...)
	if (is.nullPtr(mx@ptr) && (!missing(data) || !is.empty(mx@.dim) && !missing(fields)))
	{
		dim <- mx@.dim
		if (!missing(data))
		{
			dd <- dim.vector(data)
			if (missing(fields))
			{
				if (is.vector(data) && !is.null(n <- names(data)))
					fields <- n
				else if (!is.null(dn <- dimnames(data)) && 
					((n <- match("field", names(dn), 0)) || length(n <- which(!sapply(dn, is.null)))))
				{
					f <- n[[1]]
					fields <- dn[[f]]
					data <- aperm(data, c(f,seq_len(length(dd))[-f]))
				}
				else
					fields <- as.character(seq_len(dd[[1]]))
			}
			if (is.empty(dim))
				dim <- dd[-1]
		}
		ptr <- .Call(RmxCreateStructArray, dim, fields)
		mx <- callNextMethod(mx, ptr=ptr)
		if (!missing(data))
			mxStructSet(mx, data)
	}
	if (!is.nullPtr(mx@ptr))
	{
		mx@fields <- .Call(RmxGetFieldNames, mx@ptr)
	}
	mx
})

setMethod("dim", "mxStructArray", function(x)
	c(length(x@fields), x@.dim))
setMethod("dim<-", "mxStructArray", 
function(x, value) {
	if (value[[1]] != length(x@fields))
		stop("number of fields must not change")
	callNextMethod(x, value[-1])
})
setMethod("dimnames", "mxStructArray", function(x)
	c(list(field = x@fields), vector('list', length(x@.dim))))

mxStructIndex <- 
function(i=1, j=1, ..., field=NULL) {
	if (missing(i)) i <- NULL
	if (missing(j)) j <- NULL
	o <- list(i,j,...)
	if (missing(field) && (is.character(o[[f <- 1]]) || is.character(o[[f <- length(o)]])))
	{
		field <- o[[f]]
		o <- o[-f]
	}
	list(as.integer(unlist(o)), field=field)
}

setMethod("[[", "mxStructArray",
function(x, i, j, ...) {
	i <- mxStructIndex(i, j, ...)
	if (is.null(i$field))
	{
		v <- vector('list', f <- length(x@fields))
		for (j in seq_len(f))
		{
			e <- .Call(RmxGetField, x@ptr, i[[1]], j)
			if (!is.null(e))
				v[[j]] <- mxArray(e)
		}
		names(v) <- x@fields
		v
	}
	else
		mxArray(.Call(RmxGetField, x@ptr, i[[1]], i$field))
})

setMethod("[[<-", "mxStructArray",
function(x, i, j, ..., value) {
	i <- mxStructIndex(i, j, ...)
	if (is.null(i$field))
		stop("must specify a field to assign")
	.Call(RmxSetField, x@ptr, i[[1]], i$field, mxArrayPtr(value))
	x
})

setAs("mxStructArray", "array",
function(from) {
	n <- length(from)
	f <- length(from@fields)
	to <- array(list(), dim=c(f,n))
	for (i in seq_len(n))
		for (j in seq_len(f))
		{
			e <- .Call(RmxGetField, from@ptr, i, j)
			if (!is.null(e))
				to[[j,i]] <- mxArray(e)
		}
	dim(to) <- dim(from)
	dimnames(to) <- dimnames(from)
	return(to)
})

setAs("array", "mxStructArray", function(from) 
	new("mxStructArray", data=from))

setMethod("[", "mxStructArray",
function(x, i, j, ..., recursive=FALSE, drop=FALSE) {
	if (!missing(i))
	{
		i <- mxStructIndex(i,j,...)
		x <- do.call(`[[`, c(x, i))
		if (is.null(x))
			return(x)
	}
	x <- as(x, "array")
	if (drop)
		x <- base::drop(x)
	if (recursive)
		x[] <- lapply(x, `[`, recursive=recursive, drop=drop)
	x
})

setMethod("[<-", "mxStructArray",
function(x, i, j, ..., value) {
	if (!missing(i))
	{
		i <- mxStructIndex(i,j,...)
		x <- do.call(`[[<-`, c(x, i, value=as.matlab(value)))
	}
	else
		mxStructSet(x, value)
	x
})

setMethod("$", "mxStructArray",
function(x, name) {
	if (x@.length != 1)
		stop("$ operator requires scalar mxStructArray")
	mxArray(.Call(RmxGetField, x@ptr, 1L, name))
})

setMethod("$<-", "mxStructArray",
function(x, name, value) {
	if (x@.length != 1)
		stop("$<- operator requires scalar mxStructArray")
	.Call(RmxSetField, x@ptr, 1L, name, mxArrayPtr(value))
	x
})

mxAddField <-
function(mx, field) {
	if (class(mx) != 'mxStructArray')
		stop('must be mxStructArray')
	if (r <- .Call(RmxAddField, mx@ptr, field))
		mx@fields[[r]] <- field
	mx
}

mxRemoveField <-
function(mx, field) {
	if (class(mx) != 'mxStructArray')
		stop('must be mxStructArray')
	.Call(RmxRemoveField, mx@ptr, field)
	mx@fields <- .Call(RmxGetFieldNames, mx@ptr)
	mx
}

setMethod("print", "mxStructArray",
function(x, ...) {
	callNextMethod(x, ...)
	cat("fields: ")
	print(x@fields, quote=FALSE)
	invisible(x)
})


#######################################
mxClass <- function(c) if (!is.null(c)) switch(as.character(c),
	cell = "Cell",
	struct = "Struct",
	logical = "Logical",
	char = "Char",
	void = "",
	double = "Numeric",
	single = "Numeric",
	int8 = "Numeric",
	uint8 = "Numeric",
	int16 = "Numeric",
	uint16 = "Numeric",
	int32 = "Numeric",
	uint32 = "Numeric",
	int64 = "Numeric",
	uint64 = "Numeric",
	`function` = "",
	opaque = "",
	object = "Struct")

mxArray <-
function(x, classID, ...) {
	if (is.null(x) || !is.mxArrayPtr(x))
		return(x)
	if (missing(classID))
		classID <- .Call(RmxGetClass, x)
	t <- mxClass(classID)
	new(paste("mx", t, "Array", sep=""), ptr=x, classID=classID, ...)
}

mxTestArray <- function() mxArray(.Call(RmxTestArray))

as.R.matlab <- 
function(x, recursive=TRUE) {
	d <- dim(x)
	t <- switch(class(x),
		NULL = return(NULL),
		mxArray = {
			warning("unsupported mxArray conversion")
			return(x)
		},
		mxCharArray = if (all(d[-2] == 1)) "character" else "data",
		mxDataArray = "data",
		mxCellArray = "data",
		mxNumericArray = "data",
		mxLogicalArray = "data",
		mxStructArray = "array",
		return(x))
	if (t == "data")
		t <- if (all(d[-1] == 1)) "vector" else "array"
	y <- as(x, t)
	if (recursive && is.recursive(y))
		y[] <- lapply(y, as.R.matlab, recursive=recursive)
	attr(y, 'mxClassID') <- x@classID
	y
}

as.matlab <-
function(x, mxClassID=attr(x, 'mxClassID')) {
	if (is.recursive(x))
		x[] <- lapply(x, as.matlab)
	if (!is.null(t <- mxClass(mxClassID)))
		new(paste("mx", t, "Array", sep=""), data=x, classID=mxClassID)
	else switch(typeof(x),
	NULL = NULL,
	list = {
		if (!is.null(rownames(x)) || (is.vector(x) && !is.null(names(x))) || 'field' %in% names(dimnames(x)))
			new('mxStructArray', data=x)
		else
			new('mxCellArray', data=x)
	},
	logical = new('mxLogicalArray', data=x),
	integer = new('mxNumericArray', data=x, classID='int32', complex=FALSE),
	double = new('mxNumericArray', data=x, classID='double', complex=FALSE),
	complex = new('mxNumericArray', data=x, classID='complex', complex=TRUE),
	character = new('mxCharArray', data=x),
	S4 = if (is.mxArray(x)) x else stop(class(x), " not convertible to matlab"),
	stop(typeof(x), " not convertible to matlab"))
}
