\name{as.matlab}
\alias{as.matlab}
\alias{as.R.matlab}
\alias{[,mxDataArray-method}
\alias{[<-,mxDataArray-method}
\alias{[,mxCellArray-method}
\alias{[<-,mxCellArray-method}
\alias{[,mxStructArray-method}
\alias{[<-,mxStructArray-method}
\title{Convert To/From Matlab Data}
\description{Marshalls matlab arrays to or from equivalent \R data structures.
If any non-trival access to or manipulation of matlab data is required, use of these functions is recommended.
}
\usage{
as.R.matlab(x, recursive=TRUE)
as.matlab(x, mxClassID=attr(x, "mxClassID"))
\S4method{[}{mxDataArray}(x, i, j, \dots, recursive=FALSE, drop=FALSE)
\S4method{[}{mxDataArray}(x, i, j, \dots) <- value
\S4method{[}{mxStructArray}(x, i, j, \dots, field=NULL, recursive=FALSE, drop=FALSE)
\S4method{[}{mxStructArray}(x, i, j, \dots, field) <- value
}
\arguments{
\item{x}{data to convert.}
\item{mxClassID}{class of mxArray to create (defaults to automatic selection).}
\item{recursive}{recursively convert all contained matlab data as well.}
\item{drop}{apply \code{drop} to the resulting array.}
\item{i,j,\dots,field}{indices of element to convert and assign just as with \code{[[}, or empty for the entire array.}
\item{value}{value to convert and assign to the specified element.}
}
\value{
\code{as.R.matlab} and \code{[} return a logical, integer, double, complex, character, list, or NULL array.

\code{as.matlab} returns a newly created \code{"mxArray"} object.
}
\details{
These functions convert between equivalent matlab and \R representations of data.

Matlab data may be a null, cell, struct, logical, char, (complex) double, (complex) single, int8, uint8, int16, uint16, or int32 array.  (Other data types are not yet supported or not representable in \R without loss of precision.)

\R data may be NULL or a logical, integer, double, complex, character, or list (containing any of these) vector, matrix, or array.

\code{as.R.matlab} always returns NULL or an array with the \code{"mxClassID"} attribute set.
This allows \code{as.matlab} to retore the original data class when passed data from \code{as.R.matlab}.

\code{[} and \code{[<-} provide simpler alternatives, combining some of the functionality of the \code{as} functions with the \code{[[} and \code{[[<-} methods.
As a special case, \code{x[]} can be used to convert or assign the entire array, and follows recycling rules on assignment.
Otherwise, indexing rules are just as with \code{[[}, and in particular only single array elements can be accessed.
}
\examples{
l <- list(a=1:3, b="abc", z=matrix(c(0,pi,pi,2*pi), 2, 2))
struct <- as.matlab(l)
struct$z[] <- c(0,2) # recycled
a <- struct[recursive=TRUE]
a[['z',1,1]][,2]
}
\keyword{interface}
\keyword{data}
