\name{matFile}
\docType{class}
\alias{matFile-class}
\alias{close.matFile}
\alias{names,matFile-method}
\alias{$,matFile-method}
\alias{$<-,matFile-method}
\alias{[[,matFile-method}
\alias{[[<-,matFile-method}
\alias{[,matFile-method}
\alias{[<-,matFile-method}
\alias{matFile}
\alias{matFile<-}
\title{Matlab MAT File Interface}
\description{
The function \code{matFile} provides a high-level interface to matlab MAT files, while
the class \code{"matFile"} provides a more direct interface.
An instance of the class represents a MAT file on disk, which can contain \code{"\linkS4class{mxArray}"} objects.
}
\section{Objects from the Class}{
Objects can be created by opening/creating a MAT file using \code{matFile(file, mode)} or \code{new("matFile", file, mode)}.
}
\section{Slots}{
Slots on matFile objects should never be modified directly, but some may be accessed:
\describe{
	\item{\code{file}:}{filename opened.}
	\item{\code{mode}:}{access mode to MAT file.}
	\item{\code{ptr}:}{\code{externalptr} containing underlying \code{MATFile*}, for access from external functions.}
}}
\usage{
matFile(file, mode, value)
# High-level interface:
matFile(file) <- value
\S4method{[}{matFile}(x,name,recursive=FALSE,drop=FALSE)
\S4method{[}{matFile}(x,name) <- value
\S3method{close}{matFile}(con)
# Low-level interface:
\S4method{names}{matFile}(x)
\S4method{[[}{matFile}(x,name)
\S4method{[[}{matFile}(x,name,global=FALSE) <- value
\S4method{$}{matFile}(x,name)
\S4method{$}{matFile}(x,name) <- value
}
\arguments{
	\item{file}{path to file to open.  Must exist when \code{mode="r"}.}
	\item{mode}{mode with which to open file, one of \code{c("r","w","u")} for read, write (truncate), or update (read-write append).  If missing, read/write entire file and close.}
	\item{value}{replacement \code{"mxArray"} value to create or replace, or \code{NULL} to delete from file, or named list of such values when writing an entire file.}
	\item{x,con}{matFile to operate on.}
	\item{name}{name of variable to operate on.}
	\item{global}{specifies whether the variable should be written as global.}
	\item{recursive,drop}{passed to value's \code{[}.}
}
\value{
\code{matFile} is a \code{"matFile"} when mode is specified, or a named list of contents of the specifed file otherwise.

\code{names} is the list of matlab variable names stored in the matFile.

\code{[[} and \code{$} return an \code{"mxArray"} object or \code{NULL} if the variable does not exist (or is the empty array).

\code{[} converts the result to an array, or if \code{name} is ommitted a list, just as with mxStructArray.
}
\details{
}
\note{
\code{"matFile"} wraps \command{MATFile*}.

\code{matFile} uses \command{matOpen}.  When reading an entire file, it calls \command{matGetNextVariable} repeatedly.  \code{close} uses \command{matClose}.

\code{[[} and related functions use \command{matGetVariable}, and \code{[[<-} uses \command{matPutVariable}, \command{matPutVariableAsGlobal}, or \command{matDeleteVariable}.
}
\seealso{
}
\examples{
file <- "tmp.mat"
matFile(file) <- list(val=1)

mat <- matFile(file, mode="u")
mat$val[[1]] <- mat$val[[1]] + 1
close(mat)

matFile(file)
file.remove(file)
}
\keyword{classes}
\keyword{file}
