\name{matlabEngine}
\docType{class}
\alias{matlabEngine-class}
\alias{matlabEngine}
\alias{close.matlabEngine}
\alias{[[,matlabEngine-method}
\alias{[[<-,matlabEngine-method}
\alias{[,matlabEngine-method}
\alias{[<-,matlabEngine-method}
\alias{$,matlabEngine-method}
\alias{$<-,matlabEngine-method}
\alias{evaluate,matlabEngine-method}
\title{Matlab Engine Interface}
\description{
This class provides an interface to matlab, allowing arbitrary matlab functions and expressions to be evaluated by running and communicating with a separate matlab process through the matlab engine library.

The function \code{matlabEngine} creates a new matlab process which can be terminated with \code{close}.
}
\section{Slots}{
Slots on matlabEngine objects should never be modified directly, but some may be accessed:
\describe{
	\item{\code{cmd}:}{command line used to start matlab.}
	\item{\code{ptr}:}{\code{externalptr} containing underlying \code{Engine*}, for access from external functions.}
}}
\usage{
matlabEngine(cmd = getOption("matlabBin"), args = c("-nosplash","-nojvm"))
\S3method{close}{matlabEngine}(con)
\S4method{[[}{matlabEngine}(x,name)
\S4method{[[}{matlabEngine}(x,name) <- value
\S4method{[}{matlabEngine}(x,name,recursive=FALSE,drop=FALSE)
\S4method{[}{matlabEngine}(x,name) <- value
\S4method{evaluate}{matlabEngine}(object, command, \dots, output=FALSE, nargout=1, tmp.var.prefix="Reval_")
\S4method{$}{matlabEngine}(x,name)
\S4method{$}{matlabEngine}(x,name) <- value
}
\arguments{
	\item{cmd}{path to matlab binary to start.}
	\item{args}{command line arguments to pass to matlab command.}
	\item{x,con,object}{matlabEngine to interface with.}
	\item{name}{name of variable to operate on.}
	\item{value}{replacement \code{"mxArray"} value to assign to variable.}
	\item{recursive,drop}{passed to value's \code{[}.}
	\item{command}{function name or matlab expression string to evaluate.}
	\item{\dots}{arguments to pass to matlab function (after passing through \code{as.matlab}).  If these are passed, it is assumed \code{command} is a simple function name, and will be assigned to variables of the form \code{paste(tmp.var.prefix, "in", 1:N, sep="")} to be passed as arguments.  If an argument is a name, the matlab variable by this name will be passed instead.}
	\item{output}{number of bytes of standard output to capture and return (\code{TRUE} defaults to a reasonable value.  By default, output generated by the command is lost.}
	\item{nargout}{number of result values to assign and return.  If this value or arguments are specified, the (first command in the) expression or function will be assigned to variables of the form \code{paste(tmp.var.prefix, "out", 1:nargout, sep="")} to be returned.}
	\item{tmp.var.prefix}{prefix for temporary matlab variable names.}
}
\value{
\code{matlabEngine} returns the new \code{"matlabEngine"} object representing the connection to the matlab process.

\code{[[} returns an \code{"mxArray"} object or \code{NULL} if the variable does not exist (or is the empty array).  \code{[} converts the result to an array.

\code{evaluate} returns the \code{nargout} \code{"mxArray"} objects resulting from the evaluation.  If \code{output} is specified, the output buffer string will be returned as the last (named \code{"output"}) item of this list.  If there is only one item to return (object or output), it will not be wrapped in a list.

\code{$} returns either the value of the named parameter if it exists, as with \code{[[}, or a function representing the named call that can be called with all the same arguments as \code{evaluate} from \code{\dots} on.
}
\details{
}
\note{
\code{"matlabEngine"} wraps \command{Engine*}.

\code{matlabEngine} uses \command{engOpen}, and \code{close} uses \command{engClose}.

\code{[[} and similar use \code{engGetVariable} and \code{[[<-} uses \code{engPutVariable}.

\code{evaluate} uses \code{engEvalString} with \code{engOutputBuffer}.
}
\seealso{
}
\examples{
eng <- matlabEngine()
eng['x'] <- as.double(3:8)
s <- eng$std(quote(x), 1)
close(eng)
}
\keyword{classes}
\keyword{interface}
\keyword{connection}
