\name{mxArray}
\docType{class}
\alias{mxArray-class}
\alias{dim,mxArray-method}
\alias{dim<-,mxArray-method}
\alias{length,mxArray-method}
\alias{mxGlobal}
\title{Matlab Array Root Class}
\description{
The class \code{"mxArray"} represents all matlab data (arrays).
Every matlab object is an instance of this class, which is a wrapper around the underlying matlab representation.
Most matlab objects are arrays, much like \R arrays, with a certain number of dimensions (at least 2) and a size for each dimension.
Most objects should not inherit directly from this class, however some unusual array types that are not representable in \R may.

In general, it is preferable to work with the equivalent \R representation of mxArray objects.  See \code{\link{as.R.matlab}} and \code{\link{as.matlab}}.
}
\section{Objects from the Class}{
Objects should not be created directly, but rather though constructors and conversion on subclasses.

Every matlab object is a type of mxArray.
Those that contain data accessible from \R are further sub-classed according to their format and contents.
Note that all of these objects are only references to the underlying data, so modifications will affect (and possibly invalidate) other references to the same data (see \code{\link{mxCopy}}).

The emtpy array (\code{[]}) is represented by \code{NULL} rather than an mxArray object.
}
\section{Slots}{
Slots on mxArray objects should never be modified directly, but some may be accessed:
\describe{
	\item{\code{classID}:}{matlab class of array, of class \code{"\linkS4class{mxClassID}"}, like \command{CLASS}.}
	\item{\code{global}:}{logical representing state of global variable flag, like \command{ISGLOBAL}.}
	\item{\code{ptr}:}{\code{externalptr} containing underlying \code{mxArray*}, for access from external functions.}
}}
\usage{
\S4method{dim}{mxArray}(x)
\S4method{dim}{mxArray}(x) <- value
\S4method{length}{mxArray}(x)
mxGlobal(mx, global)
}
\arguments{
	\item{mx,x}{mxArray to operate on.}
	\item{value}{integer vector of new array dimensions, which will be padded with \code{1}s if its length is less than 2.}
	\item{global}{logical value used to set global variable flag if specified.}
}
\value{
\code{dim}, like \command{SIZE}, returns an integer vector of dimensions of length at least 2.

\code{length}, like \command{NUMEL} (not \command{LENGTH}), returns the total number of elements.

\code{mxGlobal} returns a logical of the current setting of the variable's global workspace flag.
}
\details{
Most operations conform to the matlab semantics.  
In particular, indexing is done as with subscripts in matlab, with each index representing a 1-based offset in that dimension, wrapping into the next dimensions as indices exceed the size.
Also, \code{NA} values are in general not representable in matlab, so attempts to convert them may have undefined results.

Making an mxCellArray or mxStructArray smaller may leak non-NULL elements beyond the new end.

Objects may be coerced to \code{"raw"} to get the underlying data.
}
\note{
\code{"mxArray"} wraps \command{mxArray*}.

\code{dim} and \code{dim<-} use \command{mxGetDimensions} and \command{mxSetDimensions} respectively.

\code{length} uses \command{mxGetNumberOfElements}.

\code{mxGlobal} uses \command{mxIsFromGlobalWS} and possibly \command{mxSetFromGlobalWS}.

Coercing to \code{"raw"} returns \command{mxGetElementSize*mxGetNumberOfElements} bytes from \command{mxGetData}.
}
\seealso{
}
\keyword{interface}
\keyword{classes}
