\name{mxChar}
\docType{class}
\alias{mxCharArray-class}
\title{Matlab Character Array Class}
\description{The class \code{"mxCharArray"} represents matlab character arrays.
Character arrays are simply that: arrays of single (possibly unicode) characters.
Note that matlab handles strings very differently than \R, where character arrays are actually arrays of strings.
}
\section{Objects from the Class}{
An empty (all \code{'\0'}) array can be created by \code{new("mxCharArray", dim=dim)}.
}
\section{Extends}{
Class \code{"\linkS4class{mxDataArray}"}, directly.
Class \code{"\linkS4class{mxArray}"}, by class "mxDataArray", distance 2.
}
\details{
Because of differences between the \R and matlab representation of strings, the exact handling of mxCharArrays may depend on the content.

Single element access, and explicit coersion to/from \code{"array"} is done by treating the \R array as an array of characters, where each element has at most 1 character (\code{length(mx[[i]]) == nchar(mx[[i]]) == 1}), or 0 if the character is \code{'\0'}.
Since matlab characters may be unicode characters, this may result in truncation.

On the other hand, scalar strings in \R and simple row-vectors of characters in matlab are compatible, so coersion to/from \code{"character"} results in \code{length(to.R.matlab(mx)) == 1} and \code{nchar(to.R.matlab(mx)) == length(mx)} in this case.
Here, some unicode translation is done on the matlab side, so these lengths might not match exactly.
}
\examples{
mx <- new("mxCharArray", dim=c(1,3))
mx[[1]] <- 'a'
# 'xxx' is discarded
mx[[2]] <- 'bxxx'
mx[[1,3]] <- 'c'
# abc
as.R.matlab(mx)

dim(mx) <- c(3,1)
# now an array
as.R.matlab(mx)
}
\keyword{interface}
\keyword{data}
