\name{mxCopy}
\alias{mxCopy}
\title{Duplicate mxArray}
\description{Duplicates and copies the memory associated with an mxArray.
Usually used for the purpose of assigning a local mxArray to an element of another one which will have a shorter lifetime.
}
\usage{
mxCopy(mx)
}
\arguments{
\item{mx}{mxArray object to be copied.}
}
\details{
mxArray objects are all references.
Normally, when the \R garbage collection determines than an mxArray is no longer referenced, it will also free the underlying storage associated with it, as well as that for all mxArrays contained therein (e.g., in cells or structures).
This can present a problem in the case where an element of this mxArray is still associated with an \R symbol, either because the mxArray was subscripted or assigned to.
In this case, the element should be duplicated to prevent it being freed along with its container and the likely resultant crash.
}
\value{
\code{mxCopy} returns an mxArray that is identical to \code{mx} but that has its own copy of the underlying storage.
}
\note{Uses \command{mxDuplicateArray}.}
\examples{
## extracting an element:
x <- as.matlab(list(1:3, 'abc'))
# bad, corrupts 'y': y <- x[[2]] ; rm(x)
# copy to prevent deletion with 'x'
y <- mxCopy(x[[2]])
rm(x)
y

## replacing an element:
mxPi <- as.matlab(pi)
x <- new('mxCellArray', dim=1)
# bad, corrupts mxPi: x[[1]] <- mxPi ; rm(x)
# copy to prevent deletion with 'x'
x[[1]] <- mxCopy(mxPi)
rm(x)
mxPi
}
\keyword{interface}
\keyword{data}
