\name{mxNumeric}
\docType{class}
\alias{mxNumericArray-class}
\title{Matlab numeric Array Class}
\description{The class \code{"mxNumericArray"} represents matlab arrays of all numeric types.
These include double, single, int8, uint8, int16, uint16, int32, uint32, int64, uint64.
Of these, the last three cannot be represented in R.
Numeric arrays may be sparse and/or complex.
}
\section{Objects from the Class}{
An empty (all \code{0}) array can be created by \code{new("mxNumericArray", dim=dim, classID=class)}, where \code{class} is a \code{"mxClassID"} object or a string representation thereof.
}
\section{Slots}{
\describe{
	\item{\code{complex}:}{logical representing whether this array contains an imaginary component as well.}
	\item{\code{sparse}:}{logical representing whether this array is stored sparsely.  Sparse arrays are not yet supported by this package.}
}}
\section{Extends}{
Class \code{"\linkS4class{mxDataArray}"}, directly.
Class \code{"\linkS4class{mxArray}"}, by class "mxDataArray", distance 2.
}
\details{
Matlab supports a number of different numeric types, not all of which map to \R types directly.
In general, matlab single and \emph{double} values are convertable to \R doubles (or complex), and int8, uint8, int16, uint16, and \emph{int32} values are convertable to \R integers.
The indicated types are the default for conversions.
Currently, other matlab types may not be accessed as doing so would result in a loss of precision.

Sparse numeric arrays are not yet supported.
Only mxArray operations will succeed.
}
\examples{
mx <- new("mxNumericArray", dim=c(3,2), classID="uint8")
for (i in seq_len(length(mx))) mx[[i]] <- i
as.R.matlab(mx)
as.matlab(sin(seq(0,2*pi, length.out=50)), mxClassID="single")
}
\keyword{interface}
\keyword{data}
