\name{mxStruct}
\docType{class}
\alias{mxStructArray-class}
\alias{dim,mxStructArray-method}
\alias{dim<-,mxStructArray-method}
\alias{dimnames,mxStructArray-method}
\alias{mxAddField}
\alias{mxRemoveField}
\alias{[[,mxStructArray-method}
\alias{[[<-,mxStructArray-method}
\alias{$,mxStructArray-method}
\alias{$<-,mxStructArray-method}
\title{Matlab Structure Class}
\description{The class \code{"mxStructArray"} represents matlab struct arrays.
Structs are much like cell arrays with an extra, named dimension of fields, which for the purpose of this class is considered to be the first dimension.
}
\section{Objects from the Class}{
An empty (all \code{NULL}) struct can be created by \code{new("mxStructArray", dim=dim, fields=fields)}.
}
\section{Extends}{
Class \code{"\linkS4class{mxArray}"}, directly.
}
\section{Slots}{
\describe{
	\item{\code{fields}:}{character vector of field names, like \command{FIELDNAMES}. \code{rownames} may also be used and is preferred.}
}}
\usage{
\S4method{dim}{mxStructArray}(x)
\S4method{dim}{mxStructArray}(x) <- value
\S4method{dimnames}{mxStructArray}(x)
mxAddField(mx, field)
mxRemoveField(mx, field)
\S4method{[[}{mxStructArray}(x, i, j, \dots, field=NULL)
\S4method{[[}{mxStructArray}(x, i, j, \dots, field) <- value
\S4method{$}{mxStructArray}(x, field)
\S4method{$}{mxStructArray}(x, field) <- value
}
\arguments{
\item{mx,x}{mxStructArray object to be manipulated.}
\item{field}{name (or index number) of field to be manipulated.}
\item{i,j,\dots}{indices of element to extract or replace.  If \code{field} is not specified and the first or last index is a character, it is used as the field.  For access only, if \code{field} is \code{NULL}, a named list is returned with all fields for that index.}
\item{value}{the replacement element, which must be either \code{NULL} or \code{"mxArray"}.}
}
\details{
Structures are represented as \code{n+1}-dimensional arrays, where the first dimension is the field.
Accordingly, \code{dim} is \code{c(length(fields), SIZE)}.

The field names and count may only be modified by \code{mxAddField} and \code{mxRemoveField}.
Removing a field may leak non-NULL elements in that field.

Assigning an element will explicitly destroy the previous contents, invalidating any existing references to it.  Likewise, when an mxStructArray is no longer referenced, all its contained mxArrays are freed along with it.  If this presents a problem, see \code{\link{mxCopy}}.

\code{$} and \code{$<-} work in the obvious way on scalar mxStructArrays only.
}
\value{
\code{dimnames} associates the field names to the first dimension, so \code{rownames} is often preferable.

\code{mxAddField} and \code{mxRemoveField} return the updated \code{mx}.
The underlying data is modified in-place, so the old value of \code{mx} is invalid after the call.
}
\note{
\code{mxAddField} and \code{mxRemoveField} use the matlab functions of the same name.

\code{[[} And \code{[[<-} use \command{mxGetFieldByNumber} and \command{mxSetFieldByNumber} respectively.
}
\examples{
# create a 2-element row-vector struct array with three fields
mx <- new("mxStructArray", dim=c(1,2), fields=c("a","b","c"))

# reference mx(1).b three ways:
mx[1,field="b"] <- 3.14
mx[[1,1]]$b
mx[['b']]

mx <- mxAddField(mx, "d")
mx <- mxRemoveField(mx, "a")
# remove field "c" (the second field):
mx <- mxRemoveField(mx, 2)
rownames(mx)
}
\keyword{interface}
\keyword{data}
