#include "Rmatlab.h"
#include <mat.h>

SEXP rmatFileTag;

static inline Rboolean isRmatFile(SEXP ex)
{
	return TYPEOF(ex) == EXTPTRSXP && R_ExternalPtrTag(ex) == rmatFileTag;
}

MATFile *rmatFile(SEXP ex)
{
	return isRmatFile(ex) ? R_ExternalPtrAddr(ex) : NULL;
}

static int rmatClose(SEXP ex)
{
	MATFile *mat = rmatFile(ex);
	if (!mat)
		return 1;
	R_ClearExternalPtr(ex);
	return matClose(mat);
}

SEXP rmatFileNew(MATFile *mat)
{
	if (mat == NULL)
		return R_NilValue;
	SEXP ex = R_MakeExternalPtr(mat, rmatFileTag, R_NilValue);
	R_RegisterCFinalizerEx(ex, (void (*)(SEXP))&rmatClose, true);
	return ex;
}

SEXP RmatIsFile(SEXP ex)
{
	return ScalarLogical(isRmatFile(ex));
}

SEXP RmatClose(SEXP ex)
{
	int r = rmatClose(ex);
	return ScalarLogical(r == 0);
}

SEXP RmatOpen(SEXP file, SEXP mode)
{
	MATFile *mat = matOpen(CHAR(asChar(file)), CHAR(asChar(mode)));
	if (!mat)
		return R_NilValue;
	return rmatFileNew(mat);
}

SEXP RmatSetVariable(SEXP rmf, SEXP rname, SEXP rmx, SEXP global)
{
	MATFile *mat = rmatFile(rmf);
	if (!mat)
		error_return("matFile not open");
	mxArray *mx = rmxArray(rmx);
	const char *name = CHAR(PROTECT(asChar(rname)));
	int r = 1;
	if (mx)
		if (asLogical(global))
			r = matPutVariableAsGlobal(mat, name, mx);
		else
			r = matPutVariable(mat, name, mx);
	else
		r = matDeleteVariable(mat, name);
	UNPROTECT(1);
	return ScalarLogical(r == 0);
}

SEXP RmatGetVariable(SEXP rmf, SEXP name)
{
	MATFile *mat = rmatFile(rmf);
	if (!mat)
		error_return("matFile not open");
	mxArray *mx = matGetVariable(mat, CHAR(asChar(name)));
	return rmxArrayNew(mx, true);
}

SEXP RmatGetDir(SEXP rmf)
{
	MATFile *mat = rmatFile(rmf);
	if (!mat)
		error_return("matFile not open");
	int n = 0;
	char **dir = matGetDir(mat, &n);
	if (n < 0)
		return ScalarLogical(FALSE);
	SEXP res = PROTECT(allocVector(STRSXP, n));
	int i;
	for (i = 0; i < n; i ++)
		SET_STRING_ELT(res, i, mkChar(dir[i]));
	mxFree(dir);
	UNPROTECT(1);
	return res;
}

SEXP RmatGetNextVariable(SEXP rmf)
{
	MATFile *mat = rmatFile(rmf);
	if (!mat)
		error_return("matFile not open");
	const char *name;
	mxArray *mx = matGetNextVariable(mat, &name);
	if (!mx)
		return R_NilValue;
	SEXP rmx = PROTECT(rmxArrayNew(mx, true));
	setAttrib(rmx, install("name"), mkString(name));
	UNPROTECT(1);
	return rmx;
}
