
#include "Rmatlab.h"
#include <engine.h>

SEXP rmatlabEngineTag;

static inline Rboolean isRmatlabEngine(SEXP ex)
{
	return TYPEOF(ex) == EXTPTRSXP && R_ExternalPtrTag(ex) == rmatlabEngineTag;
}

Engine *rmatlabEngine(SEXP ex)
{
	return isRmatlabEngine(ex) ? R_ExternalPtrAddr(ex) : NULL;
}

static int rmatlabEngineClose(SEXP ex)
{
	Engine *eng = rmatlabEngine(ex);
	if (!eng)
		return 1;
	R_ClearExternalPtr(ex);
	return engClose(eng);
}

SEXP rmatlabEngineNew(Engine *eng)
{
	if (eng == NULL)
		return R_NilValue;
	SEXP ex = R_MakeExternalPtr(eng, rmatlabEngineTag, R_NilValue);
	R_RegisterCFinalizerEx(ex, (void (*)(SEXP))&rmatlabEngineClose, true);
	return ex;
}

SEXP RmatlabIsEngine(SEXP ex)
{
	return ScalarLogical(isRmatlabEngine(ex));
}

SEXP RmatlabEngineClose(SEXP ex)
{
	int r = rmatlabEngineClose(ex);
	return ScalarLogical(r == 0);
}

SEXP RmatlabEngineOpen(SEXP cmd)
{
	Engine *eng = engOpen(CHAR(asChar(cmd)));
	if (!eng)
		return R_NilValue;
	return rmatlabEngineNew(eng);
}

SEXP RmatlabEngineEval(SEXP reng, SEXP str, SEXP outlen)
{
	Engine *eng = rmatlabEngine(reng);
	if (!eng)
		error_return("matlabEngine not open");
	int len = asInteger(outlen);
	char *buf = NULL;
	if (len > 0)
	{
		buf = calloc(len+1, 1);
		engOutputBuffer(eng, buf, len);
	}
	printf("eval: %s\n", CHAR(asChar(str)));
	int r = engEvalString(eng, CHAR(asChar(str)));
	SEXP res;
	if (buf)
	{
		PROTECT(res = mkString(buf));
		engOutputBuffer(eng, NULL, 0);
		free(buf);
	}
	else
		PROTECT(res = ScalarLogical(r == 0));
	UNPROTECT(1);
	return res;
}

SEXP RmatlabEnginePutVariable(SEXP reng, SEXP name, SEXP rmx)
{
	Engine *eng = rmatlabEngine(reng);
	if (!eng)
		error_return("matlabEngine not open");
	mxArray *mx = rmxArray(rmx);
	int r = engPutVariable(eng, CHAR(asChar(name)), mx);
	return ScalarLogical(r == 0);
}

SEXP RmatlabEngineGetVariable(SEXP reng, SEXP name)
{
	Engine *eng = rmatlabEngine(reng);
	if (!eng)
		error_return("matlabEngine not open");
	mxArray *mx = engGetVariable(eng, CHAR(asChar(name)));
	return rmxArrayNew(mx, true);
}
